@@ -3,7 +3,7 @@ package YAHC;
 use strict;
 use warnings;
 
-our $VERSION = '0.014';
+our $VERSION = '0.015';
 
 use EV;
 use Time::HiRes;
@@ -53,7 +53,7 @@ our @EXPORT_OK = qw/
     yahc_conn_timeline
     yahc_conn_request
     yahc_conn_response
-    yahc_conn_retries_left
+    yahc_conn_attempts_left
 /;
 
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
@@ -134,7 +134,7 @@ sub request {
     $request->{_target} = _wrap_target_selection($request->{host}) if $request->{host};
     do { $request->{$_} ||= $pool_args->{$_} if $pool_args->{$_} } foreach (qw/host port scheme request_timeout
                                                                                connect_timeout drain_timeout/);
-    $request->{scheme} //= 'http';
+    $request->{scheme} ||= 'http';
     die "YAHC: only support scheme http\n" unless $request->{scheme} eq 'http';
     die "YAHC: host must be defined\n" unless $request->{host};
 
@@ -211,7 +211,7 @@ sub yahc_conn_errors        { $_[0]->{errors}   }
 sub yahc_conn_timeline      { $_[0]->{timeline} }
 sub yahc_conn_request       { $_[0]->{request}  }
 sub yahc_conn_response      { $_[0]->{response} }
-sub yahc_conn_retries_left  { $_[0]->{retries} - $_[0]->{attempt} }
+sub yahc_conn_attempts_left { $_[0]->{retries} - $_[0]->{attempt} + 1 }
 
 sub yahc_conn_target {
     my $target = $_[0]->{selected_target};
@@ -624,7 +624,9 @@ sub _set_until_state_timer {
     };
 
     _register_in_timeline($conn, "setting $timeout_name to %.3fs", $timeout) if $conn->{keep_timeline};
-    $watchers->{$timer_name} = $self->{loop}->timer($timeout, 0, $timer_cb);
+    my $w = $watchers->{$timer_name} = $self->{loop}->timer_ns($timeout, 0, $timer_cb);
+    $w->priority(-2); # set lowest priority
+    $w->start;
 }
 
 ################################################################################
@@ -751,7 +753,9 @@ sub _strstate {
 
 sub _log_message {
     my $format = shift;
-    printf STDERR "[%s] [$$] $format\n", POSIX::strftime('%F %T', localtime), @_;
+    my $now = Time::HiRes::time;
+    my ($sec, $ms) = split(/[.]/, $now);
+    printf STDERR "[%s.%05d] [$$] $format\n", POSIX::strftime('%F %T', localtime($now)), $ms || 0, @_;
 }
 
 1;
